#include "peripherals.h"

#include "boards.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#include "TT_Config.h"


/* DEFINES */
#define SAADC_TIMER_INSTANCE    1
#define EEPROM_TWI_INSTANCE     1
#define OLED_SPI_INSTANCE       0

/* TIMER instances */
static const nrf_drv_timer_t SAADC_TIMER = NRF_DRV_TIMER_INSTANCE(SAADC_TIMER_INSTANCE);          /**< ADC timer. */
static const nrf_drv_timer_t BATTERY_TIMER = NRF_DRV_TIMER_INSTANCE(2);                           /**< Battery timer. */

/* PPI instances */
static nrf_ppi_channel_t SAADC_TIMER_PPI_CHANNEL;

/* TWI instances */
const nrf_drv_twi_t m_eeprom_twi = NRF_DRV_TWI_INSTANCE(EEPROM_TWI_INSTANCE);

/* SPI instances */
const nrf_drv_spi_t m_oled_spi   = NRF_DRV_SPI_INSTANCE(OLED_SPI_INSTANCE);

static nrf_saadc_value_t m_buffer_pool[2][RAW_ADC_BUFF_SIZE];

uint32_t m_saadc_evt_counter;					    
bool     m_saadc_calibrate  = false;				    /**< SAADC internal calibration trigger flag */

void saadc_set_buffer(void)
{
    APP_ERROR_CHECK(nrf_drv_saadc_buffer_convert(m_buffer_pool[0], RAW_ADC_BUFF_SIZE));
    APP_ERROR_CHECK(nrf_drv_saadc_buffer_convert(m_buffer_pool[1], RAW_ADC_BUFF_SIZE));
}

void saadc_init(nrf_drv_saadc_event_handler_t battery_event_handler)
{
    nrf_saadc_channel_config_t adc_channel_config = NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN1);
    nrf_saadc_channel_config_t battery_channel_config = NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN2);

    m_saadc_calibrate = false;

    APP_ERROR_CHECK(nrf_drv_saadc_init(NULL, battery_event_handler));

    APP_ERROR_CHECK(nrf_drv_saadc_channel_init(0, &adc_channel_config));
    APP_ERROR_CHECK(nrf_drv_saadc_channel_init(1, &battery_channel_config));

    APP_ERROR_CHECK(nrf_drv_saadc_buffer_convert(m_buffer_pool[0], RAW_ADC_BUFF_SIZE));
    APP_ERROR_CHECK(nrf_drv_saadc_buffer_convert(m_buffer_pool[1], RAW_ADC_BUFF_SIZE));
}


/**
 * @brief TWI initialization.
 */
void twi_init(void)
{
    const nrf_drv_twi_config_t twi_config = {
       .scl                = ARDUINO_SCL_PIN,
       .sda                = ARDUINO_SDA_PIN,
       .frequency          = NRF_DRV_TWI_FREQ_100K,
       .interrupt_priority = APP_IRQ_PRIORITY_HIGH,
       .clear_bus_init     = true
    };

#if TWI_USE_INTERRUPT
    APP_ERROR_CHECK(nrf_drv_twi_init(&m_eeprom_twi, &twi_config, eeprom_callback, NULL));
#else
    APP_ERROR_CHECK(nrf_drv_twi_init(&m_eeprom_twi, &twi_config, NULL, NULL));
#endif
    nrf_drv_twi_enable(&m_eeprom_twi);
}


/**
 * @brief SPI initialization.
 */
void spi_init(void)
{
    nrf_drv_spi_config_t spi_config = NRF_DRV_SPI_DEFAULT_CONFIG;
    spi_config.ss_pin   = ARDUINO_10_PIN;
//    spi_config.miso_pin = ARDUINO_12_PIN;
    spi_config.mosi_pin = ARDUINO_11_PIN;
    spi_config.sck_pin  = ARDUINO_13_PIN;

#if SPI_USE_INTERRUPT
    APP_ERROR_CHECK(nrf_drv_spi_init(&m_oled_spi, &spi_config, oled_spi_callback, NULL));
#else
    APP_ERROR_CHECK(nrf_drv_spi_init(&m_oled_spi, &spi_config, NULL, NULL));
#endif      
}


/**@brief Function for the Timer initialization.
 *
 * @details Initializes the timer module. This creates and starts application timers.
 */
void timers_init(nrfx_timer_event_handler_t saadc_timer, nrfx_timer_event_handler_t battery_timer_handler)
{
    nrf_drv_timer_config_t timer_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;

    /* Enable app timer */
    APP_ERROR_CHECK(app_timer_init());

    /* Create ADC timer */
    APP_ERROR_CHECK(nrf_drv_timer_init(&SAADC_TIMER, &timer_cfg, saadc_timer));
    nrf_drv_timer_extended_compare(&SAADC_TIMER, NRF_TIMER_CC_CHANNEL0, nrf_drv_timer_ms_to_ticks(&SAADC_TIMER, ADC_ACQUISITION_INTERVAL), NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, true);

    /* Configure BATTERY TIMER */
    APP_ERROR_CHECK(nrf_drv_timer_init(&BATTERY_TIMER, &timer_cfg, battery_timer_handler));
    nrf_drv_timer_extended_compare(&BATTERY_TIMER, NRF_TIMER_CC_CHANNEL0, nrf_drv_timer_ms_to_ticks(&BATTERY_TIMER, BATTERY_LEVEL_MEAS_INTERVAL), NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, true);
}


void saadc_sampling_event_init(void)
{
    uint32_t transducer_timer_compare_event_addr, battery_timer_compare_event_addr;
    uint32_t saadc_sample_task_addr;

    APP_ERROR_CHECK(nrf_drv_ppi_init());

    nrf_drv_timer_enable(&SAADC_TIMER);
    nrf_drv_timer_enable(&BATTERY_TIMER);

    transducer_timer_compare_event_addr = nrf_drv_timer_compare_event_address_get(&SAADC_TIMER, NRF_TIMER_CC_CHANNEL0);
    
    saadc_sample_task_addr = nrf_drv_saadc_sample_task_get();
    /* Setup ppi channel so that timer compare event is triggering sample tasks in SAADC */
    APP_ERROR_CHECK(nrf_drv_ppi_channel_alloc(&SAADC_TIMER_PPI_CHANNEL));

    APP_ERROR_CHECK(nrf_drv_ppi_channel_assign(SAADC_TIMER_PPI_CHANNEL, transducer_timer_compare_event_addr, saadc_sample_task_addr));
}


void saadc_sampling_event_enable(void)
{
    APP_ERROR_CHECK(nrf_drv_ppi_channel_enable(SAADC_TIMER_PPI_CHANNEL));
}

void saadc_wait_calibration(void)
{
    if (true == m_saadc_calibrate)
    {
	NRF_LOG_INFO("SAADC calibration starting...  \r\n");

	/* Trigger calibration task */
        while (NRF_SUCCESS != nrf_drv_saadc_calibrate_offset())
	{
	    
	}

        m_saadc_calibrate = false;
    }
}